# Local addition to autoconf macros.
# Copyright (C) 1998
# Martin Gall

# Local additions to Autoconf macros.
# Copyright (C) 1992, 1994 Free Software Foundation, Inc.
# Francois Pinard <pinard@iro.umontreal.ca>, 1992.

# @defmac AC_PROG_CC_STDC
# @maindex PROG_CC_STDC
# @ovindex CC
# If the C compiler in not in ANSI C mode by default, try to add an option
# to output variable @code{CC} to make it so.  This macro tries various
# options that select ANSI C on some system or another.  It considers the
# compiler to be in ANSI C mode if it defines @code{__STDC__} to 1 and
# handles function prototypes correctly.
# 
# If you use this macro, you should check after calling it whether the C
# compiler has been set to accept ANSI C; if not, the shell variable
# @code{ac_cv_prog_cc_stdc} is set to @samp{no}.  If you wrote your source
# code in ANSI C, you can make an un-ANSIfied copy of it by using the
# program @code{ansi2knr}, which comes with Ghostscript.
# @end defmac

define(FP_PROG_CC_STDC,
[AC_MSG_CHECKING(for ${CC-cc} option to accept ANSI C)
AC_CACHE_VAL(ac_cv_prog_cc_stdc,
[ac_cv_prog_cc_stdc=no
ac_save_CFLAGS="$CFLAGS"
# Don't try gcc -ansi; that turns off useful extensions and
# breaks some systems' header files.
# AIX			-qlanglvl=ansi
# Ultrix and OSF/1	-std1
# HP-UX			-Aa -D_HPUX_SOURCE
# SVR4			-Xc
for ac_arg in "" -qlanglvl=ansi -std1 "-Aa -D_HPUX_SOURCE" -Xc
do
  CFLAGS="$ac_save_CFLAGS $ac_arg"
  AC_TRY_COMPILE(
[#if !defined(__STDC__) || __STDC__ != 1
choke me
#endif	
], [int test (int i, double x);
struct s1 {int (*f) (int a);};
struct s2 {int (*f) (double a);};],
[ac_cv_prog_cc_stdc="$ac_arg"; break])
done
CFLAGS="$ac_save_CFLAGS"
])
AC_MSG_RESULT($ac_cv_prog_cc_stdc)
case "x$ac_cv_prog_cc_stdc" in
  x|xno) ;;
  *) CC="$CC $ac_cv_prog_cc_stdc" ;;
esac
])

# Check for function prototypes.

AC_DEFUN(FP_C_PROTOTYPES,
[AC_REQUIRE([FP_PROG_CC_STDC])
AC_MSG_CHECKING([for function prototypes])
if test "$ac_cv_prog_cc_stdc" != no; then
  AC_MSG_RESULT(yes)
  AC_DEFINE(HAVE_PROTOTYPES)
  U= ANSI2KNR=
else
  AC_MSG_RESULT(no)
  U=_ ANSI2KNR=ansi2knr
fi
AC_SUBST(U)dnl
AC_SUBST(ANSI2KNR)dnl
])

dnl
dnl Look for libpcap in .. 
dnl Use the installed libpcap if there is no local version
dnl Taken from AC_LBL_LIBPCAP
dnl
dnl usage:
dnl
dnl	MG_LIBPCAP(pcap_dep,pcap_inclpath,pcap_libpath)
dnl
AC_DEFUN(MG_LIBPCAP,
    [AC_MSG_CHECKING(for local pcap library)
    libpcap=FAIL
    lastdir=FAIL
    places=`ls .. | sed -e 's,/$,,' -e 's,^,../,' | \
	egrep '/libpcap-[[0-9]]*\.[[0-9]]*(\.[[0-9]]*)?([[ab]][[0-9]]*)?$'`
    for dir in $places ../libpcap libpcap ; do
	    basedir=`echo $dir | sed -e 's/[[ab]][[0-9]]*$//'`
	    if test $lastdir = $basedir ; then
		    dnl skip alphas when an actual release is present
		    continue;
	    fi
	    lastdir=$dir
	    if test -r $dir/pcap.c ; then
		    libpcap=$dir/libpcap.a
		    d=$dir
		    dnl continue and select the last one that exists
	    fi
    done
    if test $libpcap = FAIL ; then
	    AC_MSG_RESULT(not found)
	    AC_CHECK_LIB(pcap, main, libpcap="-lpcap")
	    if test $libpcap = FAIL ; then
		    AC_MSG_ERROR(see the INSTALL doc for more info)
	    fi
    else
	    $1=$libpcap
	    $2=$d
	    $3=$d
	    AC_MSG_RESULT($libpcap)
    fi])

# Check for ansi cpp

AC_DEFUN(MG_C_ANSI_CPP,
[AC_MSG_CHECKING([for ansi cpp])
AC_CACHE_VAL(ac_cv_c_ansi_cpp,
[AC_TRY_RUN(
[#define QUOTE(expr) #expr
int main()
{
exit(strcmp("a",QUOTE(a)));
}],ac_cv_c_ansi_cpp=yes,ac_cv_c_ansi_cpp=no,:)
])
AC_MSG_RESULT($ac_cv_c_ansi_cpp)
if test "$ac_cv_c_ansi_cpp" != no
then
	AC_DEFINE(HAVE_ANSI_CPP)
fi
])

AC_DEFUN(MG_WITH_EXTRA_INCLUDE,
[AC_MSG_CHECKING(for extra include)
AC_ARG_WITH(extra_include,
[  --with-extra-include=DIR
                          add an extra include dir],
[if test ! -z "$withval"; then
  AC_MSG_RESULT($withval)
  INCLUDES="$INCLUDES -I$withval"
else
  AC_MSG_RESULT(none)
fi], [AC_MSG_RESULT(none)])])

# Check for extra libpath

AC_DEFUN(MG_WITH_EXTRA_LIBPATH,
[AC_MSG_CHECKING(for extra libpath)
AC_ARG_WITH(extra_libpath,
[  --with-extra-libpath=DIR
                          add an extra libpath],
[if test ! -z "$withval"; then
  AC_MSG_RESULT($withval)
  LDFLAGS="$LDFLAGS -L$withval"
else
  AC_MSG_RESULT(none)
fi], [AC_MSG_RESULT(none)])])

# check for pcap

AC_DEFUN(MG_WITH_PCAP_INCLUDE,
[AC_MSG_CHECKING(pcap include location)
AC_ARG_WITH(pcap_include,
[  --with-pcap-include		libpcap include location])
if test ! -z "$with_pcap_include" ; then
 	AC_MSG_RESULT($with_pcap_include)
	INCLUDES="$INCLUDES -I$with_pcap_include"
else
	AC_MSG_RESULT(none)
fi
],)])

AC_DEFUN(MG_WITH_PCAP_LIBRARY,
[AC_MSG_CHECKING(pcap library location)
AC_ARG_WITH(pcap_library,
[  --with-pcap-library		libpcap library location])
if test ! -z "$with_pcap_library" ; then
 	AC_MSG_RESULT($with_pcap_library)
	LDFLAGS="$LDFLAGS -L$with_pcap_library"
else
	AC_MSG_RESULT(none)
fi
],)])

# Check for mg_kludge

AC_DEFUN(MG_WITH_MG_KLUDGE,
[AC_MSG_CHECKING(if mg_kludge is wanted)
AC_ARG_WITH(mg_kludge,
[  --with-mg-kludge            compile with MG_KLUDGE],
[if test "$withval" = yes; then
  AC_MSG_RESULT(yes)
  AC_DEFINE(WITH_MG_KLUDGE)
else
  AC_MSG_RESULT(no)
fi], [AC_MSG_RESULT(no)])])


# Check for pf device
AC_DEFUN(MG_SYS_PF,
[AC_MSG_CHECKING([for alpha packetfilter])
AC_CACHE_VAL(ac_cv_sys_pf,
[
if test -d /dev/pf 
then
	ac_cv_sys_pf=yes
else
	ac_cv_sys_pf=no
fi
])
AC_MSG_RESULT($ac_cv_sys_pf)
if test "$ac_cv_sys_pf" != no
then
	AC_DEFINE(HAVE_PF)
fi
])

# Check for ip_hdrincl
AC_DEFUN(MG_SYS_IP_HDRINCL,
[AC_MSG_CHECKING([for ip_hdrincl])
AC_CACHE_VAL(ac_cv_sys_ip_hdrincl,
[
AC_TRY_COMPILE(
[
#include <netinet/in.h>
#if !defined IP_HDRINCL
choke me
#endif
],
[],
ac_cv_sys_ip_hdrincl=yes,ac_cv_sys_ip_hdrincl=no)
])
AC_MSG_RESULT($ac_cv_sys_ip_hdrincl)
if test "$ac_cv_sys_ip_hdrincl" != no
then
	AC_DEFINE(HAVE_IP_HDRINCL)
fi
])

# Check for nit
AC_DEFUN(MG_SYS_NIT,
[AC_MSG_CHECKING([for sunos network interface tap])
AC_CACHE_VAL(ac_cv_sys_nit,
[
if test -d /dev/nit 
then
	ac_cv_sys_nit=yes
else
	ac_cv_sys_nit=no
fi
])
AC_MSG_RESULT($ac_cv_sys_nit)
if test "$ac_cv_sys_nit" != no
then
	AC_DEFINE(HAVE_NIT)
fi
])

# Check for sock_packet
AC_DEFUN(MG_SYS_SOCK_PACKET,
[AC_MSG_CHECKING([for linux sock packet])
AC_CACHE_VAL(ac_cv_sys_sock_packet,
[
AC_EGREP_HEADER(sockaddr_pkt,linux/if_packet.h,
ac_cv_sys_sock_packet=yes,ac_cv_sys_sock_packet=no)
])
AC_MSG_RESULT($ac_cv_sys_sock_packet)
if test "$ac_cv_sys_sock_packet" != no
then
	AC_DEFINE(HAVE_SOCK_PACKET)
fi
])

# Check for siocphysaddr
AC_DEFUN(MG_SYS_SIOCPHYSADDR,
[AC_MSG_CHECKING([for sioc?physaddr])
AC_CACHE_VAL(ac_cv_sys_siocphysaddr,
[
AC_TRY_COMPILE(
[
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#if !defined SIOCRPHYSADDR || !defined SIOCSPHYSADDR
choke me
#endif
],
[],
ac_cv_sys_siocphysaddr=yes,ac_cv_sys_siocphysaddr=no)
])
AC_MSG_RESULT($ac_cv_sys_siocphysaddr)
if test "$ac_cv_sys_siocphysaddr" != no
then
	AC_DEFINE(HAVE_SIOCPHYSADDR)
fi
])

# Check for siocifhwaddr
AC_DEFUN(MG_SYS_SIOCIFHWADDR,
[AC_MSG_CHECKING([for sioc?ifhwaddr])
AC_CACHE_VAL(ac_cv_sys_siocifhwaddr,
[
AC_TRY_COMPILE(
[
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#if !defined SIOCGIFHWADDR || !defined SIOCSIFHWADDR
choke me
#endif
],
[],
ac_cv_sys_siocifhwaddr=yes,ac_cv_sys_siocifhwaddr=no)
])
AC_MSG_RESULT($ac_cv_sys_siocifhwaddr)
if test "$ac_cv_sys_siocifhwaddr" != no
then
	AC_DEFINE(HAVE_SIOCIFHWADDR)
fi
])

