/*
 * This file is a part of the mg project.
 * Copyright (C) 1998 Martin Gall
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
/*
 *
 */

#include "a.h"

char		*err_a_list[] = 
{
  "Error 0",					/* 0 */
  "Buffer has a zero length",			/* 1 */
  "Truncated",					/* 2 */
  "Buffer overflow",				/* 3 */
  "Syntax error",				/* 4 */
  "Invalid argument",				/* 5 */
  "Bad ASCII vector",				/* 6 */
  "Already exists",				/* 7 */
  "Break",					/* 8 */
  "Not found",					/* 9 */
  "Objects are identical",		        /* 10 */
  "No such key",				/* 11 */
  "No entry",					/* 12 */
  "No such base",				/* 13 */
  "Unrecognized base",				/* 14 */
  "Need more arguments",			/* 15 */
  "Too many arguments",				/* 16 */
  "Bad match",					/* 17 */
  "Bad format",					/* 18 */
  "Division by zero",				/* 19 */
  "Read error",					/* 20 */
  "Memory problem",				/* 21 */
  "Bad boolean syntax",				/* 22 */
  "Printf prematured end",			/* 23 */
  "Printf bad format",				/* 24 */
  "Printf null string",				/* 25 */
};	

t_id		*err_id = NULL;		

t_status	err_init(VOID_DECL)
{
  t_status	status;
  int		i;

  if ((err_id = id_new(HASH_SMALL_BASE,
		       VEC_ONE_BASE,
		       alloc_algorithm_factor2,
		       alloc_malloc,
		       realloc_realloc,
		       free_free,
		       "err",
		       &status)) == NULL)
    return (status);
  i = 0;
  while (i < ARRAY_COUNT(err_a_list))
    {
      if ((status = id_add(err_id,
			   (VOID_PTR)(ERR_A_START + i),
			   err_a_list[i])) < 0)
	return (status);
      i++;
    }
  return (0);
}	

/* gets the error message associated with errcode. 
   Note: errcode must be positive.
   Returns string or NULL */
char		*err_msg(errcode)
t_status	errcode;
{
  t_hash_elt	*he;

  assert(err_id);
  assert(errcode >= 0);
  if (errcode >= ERR_MAGIC_SYS)
    errcode -= ERR_MAGIC_SYS;
  if ((he = id_get(err_id,
		   (VOID_PTR)errcode)) == NULL)
    return (NULL);
  return (he->value);
}

/* prints an error message on stderr.
   The output string will look like:
   user-error-msg: a-error-msg[: sys-error-msg]
   Note that user-error-msg is issued from a vsnprintf(3), a-error-msg is
   issued from a err_msg(3), sys-error-msg by strerror(3) */
#ifdef HAVE_STDARG_H
VOID_FUNC	err_print(int errcode,char *fmt,...)
#else
VOID_FUNC	err_print(errcode,fmt,va_alist)
t_status	errcode;
char		*fmt;
va_dcl
#endif
{
  va_list	ap;
  char		buf[BUFSIZ];
  char		*err;

  assert(errcode >= 0);
  assert(fmt);
#ifdef HAVE_STDARG_H
  va_start(ap,fmt);
#else
  va_start(ap);
  va_arg(ap,int);
  va_arg(ap,char *);
#endif
  vsnprintf(buf,sizeof (buf),fmt,ap);
  if (err = err_msg(errcode))
    fprintf(stderr,"%s: %s (%d)",buf,err,errcode);
  else
    fprintf(stderr,"%s: (%d)",buf,errcode);
  if (ERR_LOOKUP_ERRNO(errcode))
    fprintf(stderr,": %s\n",strerror(errno));
  else
    fprintf(stderr,"\n");
  va_end(ap);
}

