/*
 * This file is a part of the mg project.
 * Copyright (C) 1998 Martin Gall
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
/*
 *
 */

#ifndef PAT_BOOTP_H
#define PAT_BOOTP_H	1

#include "pat.h"
#include "pat_inaddr.h"

typedef struct		s_bootp
{
  t_u8			op;
  t_u8			htype;
  t_u8			hlen;
  t_u8			hops;
  t_u32			xid;
  t_u16			secs;
  t_u16			unused;
  t_in_addr		caddr;
  t_in_addr		yaddr;
  t_in_addr		saddr;
  t_in_addr		gaddr;
  t_u8			chaddr[16];
  t_u8			sname[64];
  t_u8			file[128];
  t_u8			vendor[64];
} t_bootp;

#define BOOTP_HLEN	(sizeof (t_bootp))

#define BOOTPS_PORT	67
#define BOOTPC_PORT	68

#define PAT_BOOTP_CHECK(bootppar,bufpar,lenpar) \
	if ((lenpar) < BOOTP_HLEN)\
		return (-ERR_PAT_TRUNC);\
	(bootppar) = (t_bootp *)(bufpar);

extern t_boolean	pat_bootp_rfc2132;

#define RFC2132OPT_PAD	0
#define RFC2132OPT_END	255

extern t_pat		rfc2132optdata_pat;
extern t_pat		rfc2132opt_pat;
extern t_pat		bootp_pat;

/* PROTO pat_bootp.c */
/* pat_bootp.c */
t_status rfc2132optdata_pat_name P((void *data, t_id *optional_id, char *str, int max_len));
t_status rfc2132optdata_pat_get_tmpl P((void *data, t_id *id, char *buf, int len, char *str, int max_len));
t_field *rfc2132opt_get_fields P((char *buf, int len, t_status *status));
char *rfc2132opt_get_itmpl P((char *buf, int len, t_status *status));
t_status rfc2132opt_pat_name P((void *data, t_id *optional_id, char *str, int max_len));
t_status rfc2132opt_pat_off P((void *data, char *buf, int len, t_off *off_return));
t_status rfc2132opt_pat_sub P((void *data, t_id *optional_id, char *buf, int len, t_pat **pat_return));
t_status rfc2132opt_pat_get_field P((void *data, t_id *optional_id, char *buf, int len, char *field, char *str, int max_len));
t_status rfc2132opt_pat_set_field P((void *data, t_id *optional_id, char *buf, int len, char *field, char *value));
t_status rfc2132opt_pat_get_field_pat P((void *data, t_id *optional_id, char *buf, int len, char *field, t_pat **pat_return, void **data_return));
t_status rfc2132opt_pat_get_tmpl P((void *data, t_id *id, char *buf, int len, char *str, int max_len));
t_status rfc2132opt_pat_has_opt P((void *data, char *buf, int len, t_boolean *has_opt_return, t_off *opt_off_return, int *opt_len_return, t_pat **opt_pat_return));
t_status bootp_pat_name P((void *data, t_id *optional_id, char *str, int max_len));
t_status bootp_pat_get_field P((void *data, t_id *optional_id, char *buf, int len, char *field, char *str, int max_len));
t_status bootp_pat_set_field P((void *data, t_id *optional_id, char *buf, int len, char *field, char *value));
t_status bootp_pat_get_fields P((void *data, t_vec *vec_str));
t_status bootp_pat_get_field_pat P((void *data, t_id *optional_id, char *buf, int len, char *field, t_pat **pat_return, void **data_return));
t_status bootp_pat_get_tmpl P((void *data, t_id *id, char *buf, int len, char *str, int max_len));
t_status bootp_pat_off P((void *data, char *buf, int len, t_off *off_return));
t_status bootp_pat_sub P((void *data, t_id *optional_id, char *buf, int len, t_pat **pat_return));
t_status bootp_pat_has_opt P((void *data, char *buf, int len, t_boolean *has_opt_return, t_off *opt_off_return, int *opt_len_return, t_pat **opt_pat_return));
#endif
