/*
 * This file is a part of the mg project.
 * Copyright (C) 1998 Martin Gall
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
/*
 *
 */

#include "pat.h"
#include "pat_32.h"

t_u32			safe_ntohl(p_nu32)
t_u32			*p_nu32;	/* Pointer to network unsigned long */
{
  t_u32			a_u32;
  
  FBCOPY(p_nu32,&a_u32,sizeof (t_u32));
  return (UNSAFE_NTOHL(a_u32));
}

VOID_FUNC		safe_htonl(a_u32,p_nu32)
t_u32			a_u32;		/* Aligned host unsigned long */
t_u32			*p_nu32;	/* Pointer to network unsigned long */
{
  a_u32 = UNSAFE_HTONL(a_u32);  
  FBCOPY(&a_u32,p_nu32,sizeof (t_u32));
}

PAT_OFF_GENERIC(thirtytwo_pat_off,4)

int	ns32_chan;
PAT_NAME_GENERIC(ns32_pat_name,
		 (VOID_PTR)&ns32_chan,
		 "ns32")

PAT_EXTRACT_DECL(ns32_pat_extract)
{
  t_s32				s32;
	
  if (len < sizeof (s32))
    return (-ERR_PAT_TRUNC);
  FBCOPY(buf,&s32,sizeof (s32));
  s32 = UNSAFE_NTOHL(s32);
  return (long_to_str((signed long)s32,
		      pat_base,
		      str,
		      max_len));
}

PAT_INSERT_DECL(ns32_pat_insert)
{
  t_s32				s32;
  t_status			status;	
  int				basenum;
  signed long			num;
  
  if (len < sizeof (s32))
    return (-ERR_PAT_TRUNC);
  if ((status = long_from_str(value,
			      &basenum,
			      &num)) < 0)
    return (status);
  s32 = (t_s32)num;
  s32 = UNSAFE_HTONL(s32);
  FBCOPY(&s32,buf,sizeof (s32));
  return (0);
}

t_pat				ns32_pat = 
{
  ns32_pat_name,		/* t_pat_name_proc		*/
  thirtytwo_pat_off,		/* t_pat_off_proc		*/
  NULL,				/* t_pat_sub_proc		*/
  NULL,				/* t_pat_sum_proc		*/
  NULL,				/* t_pat_get_field_proc	*/
  NULL,				/* t_pat_set_field_proc	*/
  NULL,				/* t_pat_get_fields_proc	*/
  NULL,				/* t_pat_get_tmpl_proc	*/
  NULL,				/* t_pat_get_tmpl2_proc	*/
  NULL,				/* t_pat_has_opt_proc		*/
  NULL,				/* t_pat_adapt_len_proc	*/
  NULL,				/* t_pat_get_field_pat_proc	*/
  ns32_pat_extract,		/* t_pat_extract_proc		*/
  ns32_pat_insert,		/* t_pat_insert_proc		*/
  NULL,				/* t_pat_get_choices_proc	*/
};

int	nu32_chan;
PAT_NAME_GENERIC(nu32_pat_name,
		 (VOID_PTR)&nu32_chan,
		 "nu32")

PAT_EXTRACT_DECL(nu32_pat_extract)
{
  t_u32				u32;
  
  if (len < sizeof (u32))
    return (-ERR_PAT_TRUNC);
  FBCOPY(buf,&u32,sizeof (u32));
  u32 = UNSAFE_NTOHL(u32);
  return (ulong_to_str((unsigned long)u32,
		       pat_base,
		       str,
		       max_len));
}

PAT_INSERT_DECL(nu32_pat_insert)
{
  t_u32				u32;
  t_status			status;	
  int				basenum;
  unsigned long			num;
  
  if (len < sizeof (u32))
    return (-ERR_PAT_TRUNC);
  if ((status = ulong_from_str(value,
			       &basenum,
			       &num)) < 0)
    return (status);
  u32 = (t_u32)num;
  u32 = UNSAFE_HTONL(u32);
  FBCOPY(&u32,buf,sizeof (u32));
  return (0);
}

t_pat				nu32_pat = 
{
  nu32_pat_name,		/* t_pat_name_proc		*/
  thirtytwo_pat_off,		/* t_pat_off_proc		*/
  NULL,				/* t_pat_sub_proc		*/
  NULL,				/* t_pat_sum_proc		*/
  NULL,				/* t_pat_get_field_proc	*/
  NULL,				/* t_pat_set_field_proc	*/
  NULL,				/* t_pat_get_fields_proc	*/
  NULL,				/* t_pat_get_tmpl_proc	*/
  NULL,				/* t_pat_get_tmpl2_proc	*/
  NULL,				/* t_pat_has_opt_proc		*/
  NULL,				/* t_pat_adapt_len_proc	*/
  NULL,				/* t_pat_get_field_pat_proc	*/
  nu32_pat_extract,		/* t_pat_extract_proc		*/
  nu32_pat_insert,		/* t_pat_insert_proc		*/
  NULL,				/* t_pat_get_choices_proc	*/
};

int	u32_chan;
PAT_NAME_GENERIC(u32_pat_name,
		 (VOID_PTR)&u32_chan,
		 "u32")

PAT_EXTRACT_DECL(u32_pat_extract)
{
  t_u32				u32;

  if (len < sizeof (u32))
    return (-ERR_PAT_TRUNC);
  FBCOPY(buf,&u32,sizeof (u32));
  return (ulong_to_str((unsigned long)u32,
		       pat_base,
		       str,
		       max_len));
}

PAT_INSERT_DECL(u32_pat_insert)
{
  t_u32				u32;
  t_status			status;	
  int				basenum;
  unsigned long			num;

  if (len < sizeof (u32))
    return (-ERR_PAT_TRUNC);
  if ((status = ulong_from_str(value,
			       &basenum,
			       &num)) < 0)
    return (status);
  u32 = (t_u32)num;
  FBCOPY(&u32,buf,sizeof (u32));
  return (0);
}

t_pat				u32_pat = 
{
  u32_pat_name,			/* t_pat_name_proc		*/
  thirtytwo_pat_off,		/* t_pat_off_proc		*/
  NULL,				/* t_pat_sub_proc		*/
  NULL,				/* t_pat_sum_proc		*/
  NULL,				/* t_pat_get_field_proc	*/
  NULL,				/* t_pat_set_field_proc	*/
  NULL,				/* t_pat_get_fields_proc	*/
  NULL,				/* t_pat_get_tmpl_proc	*/
  NULL,				/* t_pat_get_tmpl2_proc	*/
  NULL,				/* t_pat_has_opt_proc		*/
  NULL,				/* t_pat_adapt_len_proc	*/
  NULL,				/* t_pat_get_field_pat_proc	*/
  u32_pat_extract,		/* t_pat_extract_proc		*/
  u32_pat_insert,		/* t_pat_insert_proc		*/
  NULL,				/* t_pat_get_choices_proc	*/
};

int	nu32assoc_chan;
PAT_NAME_GENERIC(nu32assoc_pat_name,
		 (VOID_PTR)&nu32assoc_chan,
		 "nu32assoc")

PAT_EXTRACT_DECL(nu32assoc_pat_extract)
{
  t_u32				u32;
  t_assoc			*assocs;
  t_assoc			*assoc;
  
  assocs = (t_assoc *)(data);
  if (len < sizeof (u32))
    return (-ERR_PAT_TRUNC);
  FBCOPY(buf,&u32,sizeof (u32));
  u32 = UNSAFE_NTOHL(u32);
  if (assoc = assoc_str_int_from_right(assocs,(int)u32))
    return (str_cat_str(str,
			max_len,
			assoc->left));
  else
    return (ulong_to_str((unsigned long)u32,
			 pat_base,
			 str,
			 max_len));
}

PAT_INSERT_DECL(nu32assoc_pat_insert)
{
  t_u32				u32;
  t_status			status;	
  t_assoc			*assocs;
  t_assoc			*assoc;
  
  assocs = (t_assoc *)(data);
  if (len < sizeof (u32))
    return (-ERR_PAT_TRUNC);
  if (assoc = assoc_str_ptr_from_left(assocs,value))
    u32 = (t_u32)(t_u32)(assoc->right);
  else
    {
      int			basenum;
      unsigned long		num;
      
      if ((status = ulong_from_str(value,
				   &basenum,
				   &num)) < 0)
	return (status);
      u32 = (t_u32)num;
    }
  u32 = UNSAFE_HTONL(u32);
  FBCOPY(&u32,buf,sizeof (u32));
  return (0);
}

PAT_GET_CHOICES_DECL(nu32assoc_pat_get_choices)
{
  t_assoc			*assocs;
  t_assoc			*assoc;
  t_status			status;

  assocs = (t_assoc *)(data);
  assoc = assocs;
  while (assoc->left)
    {
      if ((status = vec_str_add(vec_str,assoc->left)) < 0)
	return (status);
      assoc++;
    }
  return (0);
}

t_pat				nu32assoc_pat = 
{
  nu32assoc_pat_name,		/* t_pat_name_proc		*/
  thirtytwo_pat_off,		/* t_pat_off_proc		*/
  NULL,				/* t_pat_sub_proc		*/
  NULL,				/* t_pat_sum_proc		*/
  NULL,				/* t_pat_get_field_proc	*/
  NULL,				/* t_pat_set_field_proc	*/
  NULL,				/* t_pat_get_fields_proc	*/
  NULL,				/* t_pat_get_tmpl_proc	*/
  NULL,				/* t_pat_get_tmpl2_proc	*/
  NULL,				/* t_pat_has_opt_proc		*/
  NULL,				/* t_pat_adapt_len_proc	*/
  NULL,				/* t_pat_get_field_pat_proc	*/
  nu32assoc_pat_extract,	/* t_pat_extract_proc		*/
  nu32assoc_pat_insert,		/* t_pat_insert_proc		*/
  nu32assoc_pat_get_choices,	/* t_pat_get_choices_proc	*/
};

int	u32assoc_chan;
PAT_NAME_GENERIC(u32assoc_pat_name,
		 (VOID_PTR)&u32assoc_chan,
		 "u32assoc")

PAT_EXTRACT_DECL(u32assoc_pat_extract)
{
  t_u32				u32;
  t_assoc			*assocs;
  t_assoc			*assoc;
  
  assocs = (t_assoc *)(data);
  if (len < sizeof (u32))
    return (-ERR_PAT_TRUNC);
  FBCOPY(buf,&u32,sizeof (u32));
  if (assoc = assoc_str_int_from_right(assocs,(int)u32))
    return (str_cat_str(str,
			max_len,
			assoc->left));
  else
    return (ulong_to_str((unsigned long)u32,
			 pat_base,
			 str,
			 max_len));
}

PAT_INSERT_DECL(u32assoc_pat_insert)
{
  t_u32				u32;
  t_status			status;	
  t_assoc			*assocs;
  t_assoc			*assoc;

  assocs = (t_assoc *)(data);
  if (len < sizeof (u32))
    return (-ERR_PAT_TRUNC);
  if (assoc = assoc_str_ptr_from_left(assocs,value))
    u32 = (t_u32)(t_u32)(assoc->right);
  else
    {
      int			basenum;
      unsigned long		num;
      
      if ((status = ulong_from_str(value,
				   &basenum,
				   &num)) < 0)
	return (status);
      u32 = (t_u32)num;
    }
  FBCOPY(&u32,buf,sizeof (u32));
  return (0);
}

PAT_GET_CHOICES_DECL(u32assoc_pat_get_choices)
{
  t_assoc			*assocs;
  t_assoc			*assoc;
  t_status			status;

  assocs = (t_assoc *)(data);
  assoc = assocs;
  while (assoc->left)
    {
      if ((status = vec_str_add(vec_str,assoc->left)) < 0)
	return (status);
      assoc++;
    }
  return (0);
}

t_pat				u32assoc_pat = 
{
  u32assoc_pat_name,		/* t_pat_name_proc		*/
  thirtytwo_pat_off,		/* t_pat_off_proc		*/
  NULL,				/* t_pat_sub_proc		*/
  NULL,				/* t_pat_sum_proc		*/
  NULL,				/* t_pat_get_field_proc	*/
  NULL,				/* t_pat_set_field_proc	*/
  NULL,				/* t_pat_get_fields_proc	*/
  NULL,				/* t_pat_get_tmpl_proc	*/
  NULL,				/* t_pat_get_tmpl2_proc	*/
  NULL,				/* t_pat_has_opt_proc		*/
  NULL,				/* t_pat_adapt_len_proc	*/
  NULL,				/* t_pat_get_field_pat_proc	*/
  u32assoc_pat_extract,		/* t_pat_extract_proc		*/
  u32assoc_pat_insert,		/* t_pat_insert_proc		*/
  u32assoc_pat_get_choices,	/* t_pat_get_choices_proc	*/
};

int	u32mask_chan;
PAT_NAME_GENERIC(u32mask_pat_name,
		 (VOID_PTR)&u32mask_chan,
		 "u32mask")

PAT_EXTRACT_DECL(u32mask_pat_extract)
{
  t_u32				u32;
  t_mask_def			*mask_defs;
	
  mask_defs = (t_mask_def *)(data);
  if (len < sizeof (t_u32))
    return (-ERR_PAT_TRUNC);
  FBCOPY(buf,&u32,sizeof (t_u32));
  return (mask_to_str(mask_defs,
		      u32,
		      "|",
		      pat_base,
		      str,
		      max_len));
}

PAT_INSERT_DECL(u32mask_pat_insert)
{
  t_u32				u32;
  t_mask_def			*mask_defs;
  t_vec				*vec_str;
  t_mask			mask;
  t_status			status;

  mask_defs = (t_mask_def *)(data);
  if (len < sizeof (t_u32))
    return (-ERR_PAT_TRUNC);
  if ((vec_str = PAT_VEC_NEW(&status)) == NULL)
    return (status);
  if ((status = vec_str_split(vec_str,
			      value,
			      '|')) < 0)
    {
      vec_str_delete(vec_str);
      return (status);
    }
  mask = 0;
  if ((status = mask_from_vec_str(mask_defs,
				  vec_str,
				  &mask)) < 0)
    {
      vec_str_delete(vec_str);
      return (status);
    }
  u32 = (t_u32)mask;
  FBCOPY(&u32,buf,sizeof (t_u32));
  vec_str_delete(vec_str);
  return (0);
}

t_pat				u32mask_pat = 
{
  u32mask_pat_name,		/* t_pat_name_proc		*/
  thirtytwo_pat_off,		/* t_pat_off_proc		*/
  NULL,				/* t_pat_sub_proc		*/
  NULL,				/* t_pat_sum_proc		*/
  NULL,				/* t_pat_get_field_proc	*/
  NULL,				/* t_pat_set_field_proc	*/
  NULL,				/* t_pat_get_fields_proc	*/
  NULL,				/* t_pat_get_tmpl_proc	*/
  NULL,				/* t_pat_get_tmpl2_proc	*/
  NULL,				/* t_pat_has_opt_proc		*/
  NULL,				/* t_pat_adapt_len_proc	*/
  NULL,				/* t_pat_get_field_pat_proc	*/
  u32mask_pat_extract,		/* t_pat_extract_proc		*/
  u32mask_pat_insert,		/* t_pat_insert_proc		*/
  NULL,				/* t_pat_get_choices_proc	*/
};
