/*
 * This file is a part of the mg project.
 * Copyright (C) 1998 Martin Gall
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
/*
 *
 */

#include <sys/types.h> 
#include <sys/socket.h> 
#include <sys/stat.h> 
#include <netinet/in.h>
#include <ctype.h>
#include <fcntl.h>
#include <time.h>
#include "pkt.h"
#include "pat_pat.h"
#include "pat_inaddr.h"
#include "pat_subnet.h"

t_pkt			*pkt_new(len,pat,alloc_proc,free_proc,status)
int			len;		
t_pat			*pat;
t_alloc_proc		alloc_proc;	
t_free_proc		free_proc;	
t_status		*status;	
{ 
  t_pkt			*npkt; 

  if ((npkt = alloc_proc(sizeof (t_pkt), 
			 "pkt", 
 			 "pkt_new:pkt", 
 			 status)) == NULL) 
    return (NULL); 
  if ((npkt->buf = alloc_proc(len * sizeof (char), 
 			      "pkt", 
 			      "pkt_new:buf", 
 			      status)) == NULL) 
    { 
      free_proc(npkt, 
 		"pkt", 
 		"pkt_new:pkt"); 
      return (NULL); 
    } 
  npkt->len = npkt->netlen = len; 
  npkt->pat = pat; 
  npkt->timestamp.tv_sec = 0; 
  npkt->timestamp.tv_usec = 0; 
  bzero(npkt->buf,npkt->len); 
  return (npkt); 
} 

t_status		pkt_scratch(src,dst,alloc_proc,free_proc) 
t_pkt			*src;		
t_pkt			*dst;		
t_alloc_proc		alloc_proc;	 
t_free_proc		free_proc;	 
{ 
  char			*nbuf;
  t_status		status;

  if ((nbuf = alloc_proc(src->len * sizeof (char), 
			 "pkt", 
			 "pkt_scratch:buf", 
			 &status)) == NULL) 
    return (status);  
  free_proc(dst->buf,
	    "pkt",
	    "*:buf");
  PKT_COPY(src,dst); 
  dst->buf = nbuf;
  bcopy(src->buf,dst->buf,src->len * sizeof (char)); 
  return (0);
}

t_pkt			*pkt_dup(pkt,alloc_proc,free_proc,status) 
t_pkt			*pkt;		
t_alloc_proc		alloc_proc;	 
t_free_proc		free_proc;	 
t_status		*status;	 
{ 
  t_pkt			*npkt; 

  if ((npkt = alloc_proc(sizeof (t_pkt), 
 			 "pkt", 
 			 "pkt_dup:pkt", 
 			 status)) == NULL) 
    return (NULL); 
  PKT_COPY(pkt,npkt); 
  if ((npkt->buf = alloc_proc(npkt->len * sizeof (char), 
 			      "pkt", 
 			      "pkt_dup:buf", 
 			      status)) == NULL) 
    { 
      free_proc(npkt, 
 		"pkt", 
 		"pkt_dup:pkt"); 
      return (NULL); 
    } 
  bcopy(pkt->buf,npkt->buf,npkt->len * sizeof (char)); 
  return (npkt); 
}

VOID_FUNC		pkt_delete(pkt,free_proc) 
t_pkt			*pkt;		 
t_free_proc		free_proc;	 
{ 
  free_proc(pkt->buf, 
 	    "pkt", 
 	    "*:buf"); 
  free_proc(pkt, 
 	    "pkt", 
 	    "*:pkt"); 
} 

t_status		pkt_save_raw(pkt,fname) 
t_pkt			*pkt; 
char			*fname;		 
{ 
  int			fd; 
  int			cc; 

  if ((fd = open(fname,O_WRONLY|O_CREAT,0600)) < 0) 
    return (-ERR_PAT_OPEN); 
  if ((cc = write(fd,pkt->buf,pkt->len)) != pkt->len) 
    return (-ERR_PAT_WRITE); 
  close(fd); 
  return (0); 
}

t_status		pkt_load_raw(pkt,fname,alloc_proc,free_proc) 
t_pkt			*pkt;		 
char			*fname;		 
t_alloc_proc		alloc_proc;	 
t_free_proc		free_proc;	 
{ 
  struct stat		st; 
  int			fd; 
  int			cc; 
  t_status		status; 
  char			*buf; 

  if (stat(fname,&st) < 0) 
    return (-ERR_PAT_SYSCALL); 
  if ((fd = open(fname,O_RDONLY)) < 0)
    return (-ERR_PAT_OPEN);
  if ((buf = alloc_proc(st.st_size,
			"pkt",
			"pkt_load_raw:buf",
			&status)) == NULL)
    {
      close(fd);
      return (status);
    }
  if ((cc = read(fd,buf,st.st_size)) < 0)
    {
      status = -ERR_PAT_READ;
      goto bad;
    }
  if (cc != st.st_size)
    {
      status = -ERR_PAT_TRUNC;
      goto bad;
    }
  free_proc(pkt->buf,
	    "pkt",
	    "*:buf");
  pkt->buf = buf;
  pkt->len = pkt->netlen = st.st_size;
  status = 0;
  goto end;
bad:
  free_proc(buf,
	    "pkt",
	    "pkt_load_raw:buf");
end:
  close(fd);
  return (status);
}

t_status		pkt_sub(pkt,sub_pkt,id)
t_pkt			*pkt;		
t_pkt			*sub_pkt;	
t_id			*id;		
{
  t_status		status;
  t_pat			*sub_pat;
  t_off			off;
  
  if (!pkt->pat->sub_proc)
    return (-ERR_PAT_NO_METHOD_SUB);
  if ((status = pkt->pat->sub_proc(NULL,
				   id,
				   pkt->buf,
				   pkt->len,
				   &sub_pat)) < 0)
    return (status);
  sub_pkt->pat = sub_pat;
  if (!pkt->pat->off_proc)
    return (-ERR_PAT_NO_METHOD_OFF);
  if ((status = pkt->pat->off_proc(NULL,
				   pkt->buf,
				   pkt->len,
				   &off)) < 0)
    return (status);
  if (pkt->len == off)
    {
      status = -ERR_PAT_EXACT_MATCH;
      goto dosub;
    }
  if (off == 0)
    return (-ERR_PAT_LOOP);
  if (pkt->len < off)
    return (-ERR_PAT_TRUNC);
  status = 0;
dosub:
  sub_pkt->buf = pkt->buf + off;
  sub_pkt->len = pkt->len - off;
  sub_pkt->netlen = pkt->netlen;
  sub_pkt->timestamp = pkt->timestamp;
  return (status);
}

t_status		pkt_subn_i(pkt,
				   subn_pkt,
				   wanted_name,
				   wanted_name_len,
				   do_opts,
				   id)
t_pkt			*pkt;		
t_pkt			*subn_pkt;	
char			*wanted_name;	
int			wanted_name_len; 
t_boolean		do_opts;	
t_id			*id;		
{
  t_pat			*sub_pat;
  t_status		status;
  char			name[STR_BUFSIZ];
  t_pkt			tmp_pkt;
  
  name[0] = 0;
  if (!pkt->pat->name_proc)
    return (-ERR_PAT_NO_METHOD_NAME);
  if ((status = pkt->pat->name_proc(NULL,
				    id,
				    name,
				    sizeof (name))) < 0)
    return (status);
  if (!strncmp(name,wanted_name,wanted_name_len))
    {
      PKT_COPY(pkt,subn_pkt);
      return (0);
    }
  if (do_opts)
    {
      t_boolean		has_opt;
      t_off		opt_off;
      int		opt_len;
      t_pat		*opt_pat;

      if (pkt->pat->has_opt_proc)
	{
	  if ((status = pkt->pat->has_opt_proc(NULL,
					       pkt->buf,
					       pkt->len,
					       &has_opt,
					       &opt_off,
					       &opt_len,
					       &opt_pat)) < 0)
	    return (status);
	  if (has_opt)
	    {
	      t_pkt	opt_pkt;
	      
	      if ((opt_off + opt_len) > pkt->len)
		return (-ERR_PAT_OPT_BAD_MATCH);
	      PKT_COPY(pkt,&opt_pkt);
	      opt_pkt.pat = opt_pat;
	      opt_pkt.buf = pkt->buf + opt_off;
	      opt_pkt.len = opt_len;
	      if ((status = pkt_subn_i(&opt_pkt,
				       subn_pkt,
				       wanted_name,
				       wanted_name_len,
				       do_opts,
				       id)) < 0)
		{
		  if (status != -ERR_PAT_EXACT_MATCH)
		    return (status);
		}
	      else
		return (0);
	    }
	}
    }
  if ((status = pkt_sub(pkt,
			subn_pkt,
			id)) < 0)
    return (status);
  PKT_COPY(subn_pkt,&tmp_pkt);
  return (pkt_subn_i(&tmp_pkt,
		     subn_pkt,
		     wanted_name,
		     wanted_name_len,
		     do_opts,
		     id));
}

t_status		pkt_subn(pkt,
				 subn_pkt,
				 wanted_name,
				 wanted_name_len,
				 do_opts)
t_pkt			*pkt;		
t_pkt			*subn_pkt;	
char			*wanted_name;	
int			wanted_name_len; 
t_boolean		do_opts;	
{
  t_id			*id;
  t_status		status;

  if ((id = PAT_TINY_ID_NEW(&status)) == NULL)
    return (status);
  status = pkt_subn_i(pkt,
		      subn_pkt,
		      wanted_name,
		      wanted_name_len,
		      do_opts,
		      id);
  id_delete(id);
  return (status);
}

t_status		pkt_sum(pkt,up_pkt) 
t_pkt			*pkt;
t_pkt			*up_pkt;
{ 
  t_pkt			sub_pkt; 
  t_status		status; 
  t_pkt			tmp_pkt; 

  if ((status = pkt_sub(pkt,
			&sub_pkt,
			NULL)) < 0)
    { 
      switch (status)
	{
	case -ERR_PAT_EXACT_MATCH:
	case -ERR_PAT_NO_METHOD_SUB:
	  return (0);
	default:
	  return (status);
	}
    }
  PKT_COPY(pkt,&tmp_pkt); 
  if ((status = pkt_sum(&sub_pkt,
			&tmp_pkt)) < 0) 
     	    return (status); 
  if (pkt->pat->sum_proc)
    {
      if ((status = pkt->pat->sum_proc(NULL,
				       pkt->buf,
				       pkt->len,
				       up_pkt?up_pkt->buf:NULL,
				       up_pkt?up_pkt->len:0)) < 0)
	return (status);
    }
  return (0); 
} 

t_status		pkt_adapt_len(pkt)
t_pkt			*pkt;
{
  t_pkt			sub_pkt;
  t_status		status;

  if ((status = pkt_sub(pkt,
			&sub_pkt,
			NULL)) < 0)
    {
      switch (status)
	{
	case -ERR_PAT_EXACT_MATCH:
	case -ERR_PAT_NO_METHOD_SUB:
	  return (0);
	default:
	  return (status);
	}
    }
  if ((status = pkt_adapt_len(&sub_pkt)) < 0)
    return (status);
  if (pkt->pat->adapt_len_proc)
    {
      if ((status = pkt->pat->adapt_len_proc(NULL,
					     pkt->buf,
					     pkt->len)) < 0)
	return (status);
    }
  return (0);
}

t_status		pkt_get_field_to_str_i(pkt,fq_field,str,max_len,id)
t_pkt			*pkt;		
char			*fq_field;	
char			*str;		
int			max_len;	
t_id			*id;		
{
  t_status		status;
  char			*field;
  int			patlen;
  t_pkt			sub_pkt;
  t_pkt			tmp_pkt;
  
#ifdef DEBUG
  if (PAT_VERB(VERB_PAT_PKT))
    fprintf(stderr,"getting field %s\n",fq_field);
#endif
  if (!strcmp(fq_field,"Pat"))
    {
      return (pat_to_str(pkt->pat,
			 str,
			 max_len));
    }
  if (!strcmp(fq_field,"Len"))
    {
      return (long_to_str((signed long)(pkt->len),
			  pat_base,
			  str,
			  max_len));
    }
  if (!strcmp(fq_field,"Netlen"))
    {
      return (long_to_str((signed long)(pkt->netlen),
			  pat_base,
			  str,
			  max_len));
    }
  if (!strcmp(fq_field,"timestamp") || !strcmp(fq_field,"Timestamp"))
    {
      return (timeval_to_str(&pkt->timestamp,
			     (int)pat_zone,
			     str,
			     max_len,
			     fq_field[0] == 'T'));
    }
  if ((field = index(fq_field,'.')) == NULL)
    return (-ERR_PAT_NO_SUCH_FIELD);
  patlen = field - fq_field;
  field++;
  if ((status = pkt_subn_i(pkt,
			   &sub_pkt,
			   fq_field,
			   patlen,
			   TRUE,
			   id)) < 0)
    return (status);
  if (!sub_pkt.pat->get_field_proc)
    return (-ERR_PAT_NO_METHOD_GET_FIELD);
  return (sub_pkt.pat->get_field_proc(NULL,
				      id,
				      sub_pkt.buf,
				      sub_pkt.len,
				      field,
				      str,
				      max_len));
}

t_status		pkt_get_field_to_str(pkt,field,str,max_len)
t_pkt			*pkt;		
char			*field;		
char			*str;		
int			max_len;	
{
  t_id			*id;
  t_status		status;

  if ((id = PAT_TINY_ID_NEW(&status)) == NULL)
    return (status);
  status = pkt_get_field_to_str_i(pkt,
				  field,
				  str,
				  max_len,
				  id);
  id_delete(id);
  return (status);
}

t_status		pkt_get_field_to_str_no_id(pkt,field,str,max_len)
t_pkt			*pkt;		
char			*field;		
char			*str;		
int			max_len;	
{
  t_status		status;

  status = pkt_get_field_to_str_i(pkt,
				  field,
				  str,
				  max_len,
				  NULL);
  return (status);
}

t_status		pkt_set_field_from_str_i(pkt,fq_field,value,id)
t_pkt			*pkt;		
char			*fq_field;	
char			*value;		
t_id			*id;		
{
  t_status		status;
  char			*field;
  t_pkt			sub_pkt;
  t_pkt			tmp_pkt;
  int			patlen;

  if (!strcmp(fq_field,"Pat"))
    {
      t_pat		*pat;

      if ((pat = pat_from_str(value,
			      &status)) == NULL)
	return (status);
      pkt->pat = pat;
      return (0);
    }
  if ((field = index(fq_field,'.')) == NULL)
    return (-ERR_PAT_NO_SUCH_FIELD);
  patlen = field - fq_field;
  field++;
  if ((status = pkt_subn_i(pkt,
			   &sub_pkt,
			   fq_field,
			   patlen,
			   TRUE,
			   id)) < 0)
    return (status);
  if (!sub_pkt.pat->set_field_proc)
    return (-ERR_PAT_NO_METHOD_SET_FIELD);
  return (sub_pkt.pat->set_field_proc(NULL,
				      id,
				      sub_pkt.buf,
				      sub_pkt.len,
				      field,
				      value));
}

t_status		pkt_set_field_from_str(pkt,field,value)
t_pkt			*pkt;		
char			*field;		
char			*value;		
{
  t_id			*id;
  t_status		status;

  if ((id = PAT_TINY_ID_NEW(&status)) == NULL)
    return (status);
  status = pkt_set_field_from_str_i(pkt,
				    field,
				    value,
				    id);
  id_delete(id);
  return (status);
}

t_status		pkt_get_tmpl_to_str_i(pkt,str,max_len,do_opts,id)
t_pkt			*pkt;		
char			*str;		
int			max_len;	
t_boolean		do_opts;		
t_id			*id;		
{
  t_status		status;
  t_status		status2;
  t_pkt			sub_pkt;
  
  if (!pkt->pat->get_tmpl_proc)
    return (-ERR_PAT_NO_METHOD_GET_TMPL);
  if ((status = pkt->pat->get_tmpl_proc(NULL,
					id,
					pkt->buf,
					pkt->len,
					str,
					max_len)) < 0)
    return (status);
  if (do_opts)
    {
      t_boolean		has_opt;
      t_off		opt_off;
      int		opt_len;
      t_pat		*opt_pat;

      if (pkt->pat->has_opt_proc)
	{
	  if ((status = pkt->pat->has_opt_proc(NULL,
					       pkt->buf,
					       pkt->len,
					       &has_opt,
					       &opt_off,
					       &opt_len,
					       &opt_pat)) < 0)
	    return (status);
	  if (has_opt)
	    {
	      t_pkt	opt_pkt;
	      
	      if ((opt_off + opt_len) > pkt->len)
		return (-ERR_PAT_OPT_BAD_MATCH);
	      PKT_COPY(pkt,&opt_pkt);
	      opt_pkt.pat = opt_pat;
	      opt_pkt.buf = pkt->buf + opt_off;
	      opt_pkt.len = opt_len;
	      if ((status = pkt_get_tmpl_to_str_i(&opt_pkt,
						  str,
						  max_len,
						  do_opts,
						  id)) < 0)
		return (status);
	    }
	}
    }
  if ((status = pkt_sub(pkt,
			&sub_pkt,
			id)) < 0)
    {
      switch (status)
	{
	case -ERR_PAT_EXACT_MATCH:
	case -ERR_PAT_NO_METHOD_SUB:
	  return (0);
	default:
	  return (status);
	}
    }
  status = pkt_get_tmpl_to_str_i(&sub_pkt,
				 str,
				 max_len,
				 do_opts,
				 id);
  if (pkt->pat->get_tmpl2_proc)
    {
      if ((status2 = pkt->pat->get_tmpl2_proc(NULL,
					      id,
					      pkt->buf,
					      pkt->len,
					      str,
					      max_len)) < 0)
	return (status2);
    }
  return (status);
}

t_status		pkt_get_tmpl_to_str(pkt,str,max_len,do_opts)
t_pkt			*pkt;		
char			*str;		
int			max_len;		
t_boolean		do_opts;	
{
  t_id			*id;
  t_status		status;

  if ((id = PAT_TINY_ID_NEW(&status)) == NULL)
    return (status);
  status = pkt_get_tmpl_to_str_i(pkt,
				 str,
				 max_len,
				 do_opts,
				 id);
  id_delete(id);
  return (status);
}

VOID_FUNC		timeval_from_str(str,tv)
char			*str;	
struct timeval		*tv;	
{
  char			*ptr;

  if (ptr = index(str,'.'))
    *ptr++ = 0;
  else
    ptr = "0";
  tv->tv_sec = atoi(str);
  tv->tv_usec = atoi(ptr);
}

t_status		timeval_to_str(tv,zone,str,max_len,resolve)
struct timeval		*tv;		
int			zone;		
char			*str;		
int			max_len;	
t_boolean		resolve;	
{
  t_status		status;
  
  if (resolve)
    {
      unsigned int	s;
      
      s = (tv->tv_sec + zone) % 86400;
      return (str_cat_fmt_va(str,
			     max_len,
			     "%02u:%02u:%02u.%06u",
			     s / 3600,
			     (s % 3600) / 60,
			     s % 60,
			     (unsigned int)(tv->tv_usec)));
    }
  else
    return (str_cat_fmt_va(str,
			   max_len,
			   "%u.%06u",
			   (unsigned int)(tv->tv_sec),
			   (unsigned int)(tv->tv_usec)));
}

t_status		local_to_str(pkt,
				     vec_prefixes,
				     str,
				     max_len,
				     remote,
				     resolve,
				     get_field_to_str_proc)
t_pkt			*pkt;			
t_vec			*vec_prefixes;	
char			*str;		
int			max_len;	
t_boolean		remote;		
t_boolean		resolve;	
t_pkt_get_field_to_str_proc	get_field_to_str_proc;
{
  char			src[STR_BUFSIZ];
  char			dst[STR_BUFSIZ];
  struct in_addr	inaddrsrc;
  struct in_addr	inaddrdst;
  t_boolean		srcislocal;
  t_boolean		dstislocal;
  struct in_addr	*wantedinaddr;
  t_status		status;

  src[0] = 0;
  if ((status = get_field_to_str_proc(pkt,
				      "ip.src",
				      src,
				      sizeof (src))) < 0)
      return (status);
  dst[0] = 0;
  if ((status = get_field_to_str_proc(pkt,
				      "ip.dst",
				      dst,
				      sizeof (dst))) < 0)
      return (status);
  if ((status = inaddr_from_str(src,&inaddrsrc,FALSE)) < 0)
    return (status);
  if ((status = inaddr_from_str(dst,&inaddrdst,FALSE)) < 0)
    return (status);
  srcislocal = FALSE;
  VEC_FOR(vec_prefixes,t_subnet *subnet)
    {
      if (is_subnet_member(&inaddrsrc,subnet))
	{
	  srcislocal = TRUE;
	  break ;
	}
    }
  VEC_ENDFOR;
  dstislocal = FALSE;
  VEC_FOR(vec_prefixes,t_subnet *subnet)
    {
      if (is_subnet_member(&inaddrdst,subnet))
	{
	  dstislocal = TRUE;
	  break ;
	}
    }
  VEC_ENDFOR;
#define REMOTESRC	(remote?&inaddrdst:&inaddrsrc)
#define REMOTEDST	(remote?&inaddrsrc:&inaddrdst)
  if (srcislocal != dstislocal)
    wantedinaddr = srcislocal?REMOTESRC:REMOTEDST;
  else
    wantedinaddr = (inaddr_cmp(&inaddrsrc,&inaddrdst) < 0)?REMOTESRC:REMOTEDST;
  return (inaddr_to_str(wantedinaddr,
			str,
			max_len,
			resolve));
}

t_status		pkt_format_do(bs,var,pfdd)
t_bridled_str		*bs;		
char			*var;		
t_pkt_format_do_data	*pfdd;	       
{
  int			varlen;
  long			signedvalue;
  t_status		status;

  varlen = strlen(var);
  if (varlen == 0)
    return (str_cat_char(bs->str,bs->max_len,'%'));
  else
    if (isdigit(var[0]))
      {
	if (var[0] == '0')
	  {
	    if (var[1] == 'x' || var[1] == 'X')
	      signedvalue = strtol(var,NULL,16);
	    else
	      signedvalue = strtol(var,NULL,8);
	  }
	else
	  signedvalue = strtol(var,NULL,10);
	return (str_cat_char(bs->str,bs->max_len,(int)signedvalue));
      }
    else
      {
	time_t		t;
	struct tm	*tmp;

	if (pfdd->vec_prefixes)
	  {
	    if (!strcmp(var,"local") || !strcmp(var,"Local"))
	      return (local_to_str(pfdd->pkt,
				   pfdd->vec_prefixes,
				   bs->str,
				   bs->max_len,
				   FALSE,
				   var[0] == 'L',
				   pfdd->get_field_to_str_proc));
	    if (!strcmp(var,"remote") || !strcmp(var,"Remote"))
	      return (local_to_str(pfdd->pkt,
				   pfdd->vec_prefixes,
				   bs->str,
				   bs->max_len,
				   TRUE,
				   var[0] == 'R',
				   pfdd->get_field_to_str_proc));
	  }
	if ((status = pfdd->get_field_to_str_proc(pfdd->pkt,
						  var,
						  bs->str,
						  bs->max_len)) < 0)
	  if (status != -ERR_PAT_NO_SUCH_FIELD)
	    return (status);
	if (pfdd->extra_vars)
	  {
	    t_hash_elt	*he;

	    if (he = dict_get(pfdd->extra_vars,var))
	      return (str_cat_str(bs->str,
				  bs->max_len,
				  he->value));
	  }
	time(&t);
	tmp = localtime(&t);
	if (!strcmp(var,"wday"))
	  return (long_to_str((signed long)(tmp->tm_wday),
			      10,
			      bs->str,
			      bs->max_len));
	if (!strcmp(var,"mon"))
	  return (long_to_str((signed long)(tmp->tm_mon),
			      10,
			      bs->str,
			      bs->max_len));
	if (!strcmp(var,"mday"))
	  return (long_to_str((signed long)(tmp->tm_mday),
			      10,
			      bs->str,
			      bs->max_len));
	if (!strcmp(var,"hour"))
	  return (long_to_str((signed long)(tmp->tm_hour),
			      10,
			      bs->str,
			      bs->max_len));
	if (!strcmp(var,"min"))
	  return (long_to_str((signed long)(tmp->tm_min),
			      10,
			      bs->str,
			      bs->max_len));
	if (!strcmp(var,"sec"))
	  return (long_to_str((signed long)(tmp->tm_sec),
			      10,
			      bs->str,
			      bs->max_len));
	if (!strcmp(var,"year"))
	  return (long_to_str((signed long)(tmp->tm_year),
			      10,
			      bs->str,
			      bs->max_len));
      }
  return (0);
}

t_status		pkt_format_permissive_do(bs,var,pfdd)
t_bridled_str		*bs;		
char			*var;		
t_pkt_format_do_data	*pfdd;
{
  t_status		status;
  
  if ((status = pkt_format_do(bs,var,pfdd)) < 0)
    {
      switch (status)
	{
	case -ERR_PAT_TRUNC:
	  return (str_cat_str(bs->str,
			      bs->max_len,
			      "|"));
	case -ERR_PAT_CANT_HANDLE:
	  return (str_cat_str(bs->str,
			      bs->max_len,
			      "?"));
	}
      return (status);
    }
  return (0);
}

t_status		pkt_format_htmlize_do(bs,var,pfdd)
t_bridled_str		*bs;		
char			*var;		
t_pkt_format_do_data	*pfdd;
{
  t_status		status;
  t_bridled_str		htmlize_bs;
  char			*ptr;

  pfdd->htmlize_str[0] = 0;
  htmlize_bs.str = pfdd->htmlize_str;
  htmlize_bs.max_len = pfdd->htmlize_max_len;
  if ((status = pfdd->tmpl_do_proc(&htmlize_bs,var,pfdd)) < 0)
    return (status);
  ptr = htmlize_bs.str;
  while (*ptr)
    {
      switch (*ptr)
	{
	case '\n':
	  if ((status = str_cat_str(bs->str,
				    bs->max_len,
				    "<br>\n")) < 0)
	    return (status);
	  break ;
	case '<':
	  if ((status = str_cat_str(bs->str,
				    bs->max_len,
				    "&lt;")) < 0)
	    return (status);
	  break ;
	case '&':
	  if ((status = str_cat_str(bs->str,
				    bs->max_len,
				    "&amp;")) < 0)
	    return (status);
	  break ;
	default:
	  if ((!isprint(*ptr)) || ((t_u8)(*ptr) > 127)) 
	    {
	      if ((status = str_cat_char(bs->str,
					 bs->max_len,
					 '.')) < 0)
		return (status);
	    }
	  else
	    {
	      if ((status = str_cat_char(bs->str,
					 bs->max_len,
					 *ptr)) < 0)
		return (status);
	    }
	}
      ptr++;
    }
  return (0);
}

t_status		pkt_format_simple(pkt,
					  tmpl_str,
					  vec_prefixes,
					  extra_vars,
					  str,
					  max_len)
t_pkt			*pkt;		
char			*tmpl_str;	
t_vec			*vec_prefixes;	
t_dict			*extra_vars;	
char			*str;		
int			max_len;	
{
  t_pkt_format_do_data	pfdd;

  pfdd.pkt = pkt;
  pfdd.vec_prefixes = vec_prefixes;
  pfdd.get_field_to_str_proc = pkt_get_field_to_str_no_id;
  pfdd.extra_vars = extra_vars;
  pfdd.tmpl_do_proc = NULL;
  pfdd.htmlize_str = NULL;
  pfdd.htmlize_max_len = 0;
  return (tmpl_str_to_str(tmpl_str,
			  (t_tmpl_do_proc)pkt_format_do,
			  &pfdd,
			  str,
			  max_len));
}

t_status		pkt_format_simple_permissive(pkt,
						     tmpl_str,
						     vec_prefixes,
						     extra_vars,
						     str,
						     max_len)
t_pkt			*pkt;		
char			*tmpl_str;	
t_vec			*vec_prefixes;	
t_dict			*extra_vars;	
char			*str;		
int			max_len;	
{
  t_pkt_format_do_data	pfdd;

  pfdd.pkt = pkt;
  pfdd.vec_prefixes = vec_prefixes;
  pfdd.extra_vars = extra_vars;
  pfdd.get_field_to_str_proc = pkt_get_field_to_str_no_id;
  pfdd.tmpl_do_proc = NULL;
  pfdd.htmlize_str = NULL;
  pfdd.htmlize_max_len = 0;
  return (tmpl_str_to_str(tmpl_str,
			  (t_tmpl_do_proc)pkt_format_permissive_do,
			  &pfdd,
			  str,
			  max_len));
}

t_status		pkt_format_html(pkt,
					tmpl_str,
					vec_prefixes,
					extra_vars,
					str,
					max_len,
					htmlize_str,
					htmlize_max_len)
t_pkt			*pkt;		
char			*tmpl_str;	
t_vec			*vec_prefixes;	
t_dict			*extra_vars;	
char			*str;		
int			max_len;	
char			*htmlize_str;	
int			htmlize_max_len;
{
  t_pkt_format_do_data	pfdd;

  pfdd.pkt = pkt;
  pfdd.vec_prefixes = vec_prefixes;
  pfdd.get_field_to_str_proc = pkt_get_field_to_str;
  pfdd.extra_vars = extra_vars;
  pfdd.tmpl_do_proc = (t_tmpl_do_proc)pkt_format_do;
  pfdd.htmlize_str = htmlize_str;
  pfdd.htmlize_max_len = htmlize_max_len;
  return (tmpl_str_to_str(tmpl_str,
			  (t_tmpl_do_proc)pkt_format_htmlize_do,
			  &pfdd,
			  str,
			  max_len));
}

t_status		pkt_format_html_permissive(pkt,
						   tmpl_str,
						   vec_prefixes,
						   extra_vars,
						   str,
						   max_len,
						   htmlize_str,
						   htmlize_max_len)
t_pkt			*pkt;		
char			*tmpl_str;	
t_vec			*vec_prefixes;	
t_dict			*extra_vars;	
char			*str;		
int			max_len;	
char			*htmlize_str;	
int			htmlize_max_len;
{
  t_pkt_format_do_data	pfdd;

  pfdd.pkt = pkt;
  pfdd.vec_prefixes = vec_prefixes;
  pfdd.extra_vars = extra_vars;
  pfdd.get_field_to_str_proc = pkt_get_field_to_str;
  pfdd.tmpl_do_proc = (t_tmpl_do_proc)pkt_format_permissive_do;
  pfdd.htmlize_str = htmlize_str;
  pfdd.htmlize_max_len = htmlize_max_len;
  return (tmpl_str_to_str(tmpl_str,
			  (t_tmpl_do_proc)pkt_format_htmlize_do,
			  &pfdd,
			  str,
			  max_len));
}

t_status		pkt_get_choices_i(pkt,
					  fq_field,
					  vec_str,
					  do_sort_return,
					  id)
t_pkt			*pkt;		
char			*fq_field;	
t_vec			*vec_str;	
t_boolean		*do_sort_return;
t_id			*id;		
{
  t_status		status;
  char			*field;
  int			patlen;
  t_pkt			sub_pkt;
  t_pkt			tmp_pkt;
  t_boolean		has_opt;
  t_off			opt_off;
  int			opt_len;
  t_pat			*opt_pat;
  
  if (!strcmp(fq_field,"Pat"))
    {
      return (pat_displayable_get_choices(vec_str));
    }
  if ((field = index(fq_field,'.')) == NULL)
    return (-ERR_PAT_NO_SUCH_FIELD);
  patlen = field - fq_field;
  field++;
  PKT_COPY(pkt,&sub_pkt);
  do 
    {
      char		name[STR_BUFSIZ];
        
      name[0] = 0;
      if (!sub_pkt.pat->name_proc)
	return (-ERR_PAT_NO_METHOD_NAME);
      if ((status = sub_pkt.pat->name_proc(NULL,
					   id,
					   name,
					   sizeof (name))) < 0)
	return (status);
      if (!strncmp(fq_field,name,patlen))
	{
	  t_pat		*field_pat;
	  VOID_PTR	field_data;

	  if (!sub_pkt.pat->get_field_pat_proc)
	    return (-ERR_PAT_NO_METHOD_GET_FIELD_PAT);
	  if ((status =
	       sub_pkt.pat->get_field_pat_proc(NULL,
					       id,
					       sub_pkt.buf,
					       sub_pkt.len,
					       field,
					       &field_pat,
					       &field_data)) < 0)
	    return (status);
	  if (!field_pat->get_choices_proc)
	    return (-ERR_PAT_NO_METHOD_GET_CHOICES);
	  return (field_pat->get_choices_proc(field_data,
					      sub_pkt.buf,
					      sub_pkt.len,
					      vec_str,
					      do_sort_return));
	}
      if (sub_pkt.pat->has_opt_proc)
	{
	  if ((status = sub_pkt.pat->has_opt_proc(NULL,
						  sub_pkt.buf,
						  sub_pkt.len,
						  &has_opt,
						  &opt_off,
						  &opt_len,
						  &opt_pat)) < 0)
	    return (status);
	  if (has_opt)
	    {
	      t_pkt	opt_pkt;
	      
	      if ((opt_off + opt_len) > sub_pkt.len)
		return (-ERR_PAT_OPT_BAD_MATCH);
	      PKT_COPY(pkt,&opt_pkt);
	      opt_pkt.pat = opt_pat;
	      opt_pkt.buf = sub_pkt.buf + opt_off;
	      opt_pkt.len = opt_len;
	      status = pkt_get_choices_i(&opt_pkt,
					 fq_field,
					 vec_str,
					 do_sort_return,
					 id);
	      if (status != -ERR_PAT_NO_SUCH_FIELD)
		return (status);
	    }
	}
      PKT_COPY(&sub_pkt,&tmp_pkt);
    } while (pkt_sub(&tmp_pkt,
		     &sub_pkt,
		     id) == 0);
  return (-ERR_PAT_NO_SUCH_FIELD);
}

t_status		pkt_get_choices(pkt,field,vec_str,do_sort_return)
t_pkt			*pkt;		
char			*field;		
t_vec			*vec_str;	
t_boolean		*do_sort_return;
{
  t_id			*id;
  t_status		status;

  if ((id = PAT_TINY_ID_NEW(&status)) == NULL)
    return (status);
  status = pkt_get_choices_i(pkt,
			     field,
			     vec_str,
			     do_sort_return,
			     id);
  id_delete(id);
  return (status);
}

t_status		pkt_trunc(pkt,name,alloc_proc,free_proc)
t_pkt			*pkt;		
char			*name;		
t_alloc_proc		alloc_proc;
t_free_proc		free_proc;
{
  t_status		status;
  t_pkt			sub_pkt;
  t_pkt			sub_sub_pkt;
  char			*nbuf;
  int			nlen;

  if ((status = pkt_subn(pkt,
			 &sub_pkt,
			 name,
			 strlen(name),
			 TRUE)) < 0)
    return (status);
  if ((status = pkt_sub(&sub_pkt,
			&sub_sub_pkt,
			NULL)) < 0)
    return (status);
  nlen = pkt->len - sub_sub_pkt.len;
  if ((nbuf = alloc_proc(nlen * sizeof (char),
			 "pkt",
			 "pkt_trunc:buf",
			 &status)) == NULL)
    return (status);
  bcopy(pkt->buf,nbuf,nlen);
  free_proc(pkt->buf,
	    "pkt",
	    "*:buf");
  pkt->buf = nbuf;
  pkt->len = nlen;
  return (0);
}

t_status		pkt_extract(pkt,name,alloc_proc,free_proc)
t_pkt			*pkt;		
char			*name;		
t_alloc_proc		alloc_proc;
t_free_proc		free_proc;
{
  t_status		status;
  char			*nbuf;
  t_pkt			sub_pkt;

  if ((status = pkt_subn(pkt,
			 &sub_pkt,
			 name,
			 strlen(name),
			 TRUE)) < 0)
    return (status);
  if ((nbuf = alloc_proc(sub_pkt.len * sizeof (char),
			 "pkt",
			 "pkt_extract:buf",
			 &status)) == NULL)
    return (status);
  free_proc(pkt->buf,
	    "pkt",
	    "*:buf");
  bcopy(sub_pkt.buf,nbuf,sub_pkt.len);
  pkt->buf = nbuf;
  pkt->len = sub_pkt.len;
  pkt->pat = sub_pkt.pat;
  return (0);
}

t_status		pkt_cut(pkt,name,cutbufpkt,fine,alloc_proc,free_proc)
t_pkt			*pkt;		
char			*name;	   
t_pkt			*cutbufpkt;	
t_boolean		fine;		
t_alloc_proc		alloc_proc;	
t_free_proc		free_proc;
{
  t_pkt			sub_pkt;
  t_status		status;
  char			*nbuf;

  if (name)
    {
      if ((status = pkt_subn(pkt,
			     &sub_pkt,
			     name,
			     strlen(name),
			     TRUE)) < 0)
	return (status);
    }
  else
    {
      FBCOPY(pkt,&sub_pkt,sizeof (sub_pkt));
    }
  if (fine)
    {
      t_off		off;
      
      if (!sub_pkt.pat->off_proc)
	return (-ERR_PAT_NO_METHOD_OFF);
      if ((status = sub_pkt.pat->off_proc(NULL,
					  sub_pkt.buf,
					  sub_pkt.len,
					  &off)) < 0)
	return (status);
      sub_pkt.len = MY_MIN(sub_pkt.len,off);
    }
  if ((nbuf = alloc_proc(sub_pkt.len * sizeof (char),
			 "pkt",
			 "pkt_cut:buf",
			 &status)) == NULL)
    return (status);
  bcopy(sub_pkt.buf,nbuf,sub_pkt.len);
  free_proc(cutbufpkt->buf,
	    "pkt",
	    "*:buf");
  cutbufpkt->pat = sub_pkt.pat;
  cutbufpkt->buf = nbuf;
  cutbufpkt->len = sub_pkt.len;
  cutbufpkt->netlen = sub_pkt.netlen;
  return (0);
}

t_status		pkt_paste(pkt,name,cutbufpkt,fine,alloc_proc,free_proc)
t_pkt			*pkt;		
char			*name;	   
t_pkt			*cutbufpkt;	
t_boolean		fine;		
t_alloc_proc		alloc_proc;
t_free_proc		free_proc;
{
  t_status		status;
  char			*nbuf;
  int			nlen;

  if (name)
    {
      t_pkt		sub_pkt;

      if ((status = pkt_subn(pkt,
			     &sub_pkt,
			     name,
			     strlen(name),
			     TRUE)) < 0)
	{
	  if (status != -ERR_PAT_EXACT_MATCH)
	    return (status);
	}
      if (fine)
	{
	  t_off		off;
	  
	  if (!sub_pkt.pat->off_proc)
	    return (-ERR_PAT_NO_METHOD_OFF);
	  if ((status = sub_pkt.pat->off_proc(NULL,
					      sub_pkt.buf,
					      sub_pkt.len,
					      &off)) < 0)
	    return (status);
	  sub_pkt.len = MY_MIN(sub_pkt.len,off);
	  if (cutbufpkt->len > sub_pkt.len)
	    {
	      return (-ERR_PAT_ROOM);
	    }
	}
      if (cutbufpkt->len > sub_pkt.len)
	{
	  nlen = pkt->len - sub_pkt.len + cutbufpkt->len;
	  if ((nbuf = alloc_proc(nlen * sizeof (char),
				 "pkt",
				 "pkt_paste:buf",
				 &status)) == NULL)
	    return (status);
	  bzero(nbuf,nlen);
	  bcopy(pkt->buf,nbuf,pkt->len);
	  free_proc(pkt->buf,
		    "pkt",
		    "*:buf");
	  pkt->buf = nbuf;
	  pkt->len = nlen;
	}
      bcopy(cutbufpkt->buf,sub_pkt.buf,cutbufpkt->len);
    }
  else
    {
      nlen = pkt->len + cutbufpkt->len;
      if ((nbuf = alloc_proc(nlen * sizeof (char),
			     "pkt",
			     "pkt_paste:buf",
			     &status)) == NULL)
	return (status);
      bcopy(pkt->buf,nbuf,pkt->len);
      free_proc(pkt->buf,
		"pkt",
		"*:buf");
      pkt->buf = nbuf;
      bcopy(cutbufpkt->buf,pkt->buf + pkt->len,cutbufpkt->len);
      pkt->len = nlen;
    }
  return (0);
}
