/*
 * This file is a part of the mg project.
 * Copyright (C) 1998 Martin Gall
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
/*
 *
 */

#include "config.h"

#ifdef HAVE_PF
#else
# include "xip.h"
# define HAVE_NO_SEND
#endif

#ifdef HAVE_PF
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/uio.h>
#include <netinet/in.h>
#include <fcntl.h>
#include "pat_ether.h"
#include "pat_arp.h"
#include "pat_ip.h"
#include "pat_data.h"
#include "pat_ethaddr.h"
#include "pat_sz.h"
#include "xippkt.h"
#include "xipvar.h"

t_ether_addr		etheralpha_ether_dst = {0x00,0x00,0xf8,0x00,0xb3,0x96};
char			etheralpha_ifname[STR_BUFSIZ] = {'t','u','0',0};

int			etheralpha_fd = -1;

/* opens named interface.
   Calls directly pfopen(2).
   Returns a file descriptor or a negative status meaning that something has 
   failed. */
int			etheralpha_open(ifname)
char			*ifname;	/* Interface name */
{
  return (pfopen(ifname,O_WRONLY));
}

/* close file descriptor */
VOID_FUNC		etheralpha_close(fd)
int			fd;
{
  close(fd);
}

/* writes out an ethernet packet.
   Returns: See writev(2). 
   
   Warning: if dhost is incorrect (see etheraddr_from_str(3)) it
   might cause panic! */
int			etheralpha_write(fd,dhost,type,databuf,datalen)
int			fd;		/* Pf file descriptor */
t_ether_addr		*dhost;		/* Destination ethernet address */
int			type;		/* Host format of ethernet type */
char			*databuf;	/* Data buffer e.g starting
					   from ip header */
int			datalen;	/* Data length */
{
  t_ether		ether;
  struct iovec		iov[2];

  assert(datalen != 0);
  FBCOPY(dhost,&(ether.dhost),sizeof (t_ether_addr));
  ether.type = htons(type);
  iov[0].iov_base = (caddr_t)(&ether);
  iov[0].iov_len = sizeof (ether);
  iov[1].iov_base = databuf;;
  iov[1].iov_len = datalen;
  return (writev(fd,iov,2));
}

/* is a t_xip_method_proc.
   Sends out an ethernet packet.
   Is able to send "ether" pkts or either "ip" or "arp" in using
   global etheralpha_ether_dst.
   Returns 0 if OK. -ERR_XIP_NO_METHOD if packet is not ether, ip or arp.
   Might return various errors */
t_status			etheralpha_method_proc_send(xmd,args)
t_xip_method_data		*xmd;
t_vec				*args;
{
  t_ether_addr			dhost;
  int				type;
  t_pkt				sub_pkt;  
  t_pkt				*sent_pkt;
  t_status			status;
  t_in_addr			dst;
  char				buf[STR_BUFSIZ];

  if (xmd->xp->pkt->pat == &ether_pat)
    {
      buf[0] = 0;
      if ((status = pkt_get_field_to_str(xmd->xp->pkt,
					 "ether[0].dhost",
					 buf,
					 sizeof (buf))) < 0)
	return (status);
      if ((status = etheraddr_from_str(&dhost,
				       buf)) < 0)
	return (status);
      buf[0] = 0;
      if ((status = pkt_get_field_to_str(xmd->xp->pkt,
					 "ether[0].type",
					 buf,
					 sizeof (buf))) < 0)
	return (status);
      type = atoi(buf);
      if ((status = pkt_sub(xmd->xp->pkt,
			    &sub_pkt,
			    NULL)) < 0)
	return (status);
      sent_pkt = &sub_pkt;
    }
  else
    {
      if (xmd->xp->pkt->pat == &ip_pat)
	{
	  type = ETHERTYPE_IP;
	  buf[0] = 0;
	  if ((status = pkt_get_field_to_str(xmd->xp->pkt,
					     "ip[0].dst",
					     buf,
					     sizeof (buf))) < 0)
	    return (status);
	}
      else
	if (xmd->xp->pkt->pat == &arp_pat)
	  {
	    type = ETHERTYPE_ARP;
	    buf[0] = 0;
	    if ((status = pkt_get_field_to_str(xmd->xp->pkt,
					       "arpethip[0].tpa",
					       buf,
					       sizeof (buf))) < 0)
	      return (status);
	  }
	else
	  return (-ERR_XIP_NO_METHOD);
      sent_pkt = xmd->xp->pkt;
      bcopy((char *)(&etheralpha_ether_dst),
	    (char *)(&dhost),
	    sizeof (t_ether_addr));
    }
  if (etheralpha_write(etheralpha_fd,
		       &dhost,
		       type,
		       sent_pkt->buf,
		       sent_pkt->len) < 0)
    return (-ERR_XIP_WRITE);
  return (0);
}

int	etheralphaifname_chan;

PAT_NAME_GENERIC(etheralphaifname_pat_name,
		 (VOID_PTR)&etheralphaifname_chan,
		 "etheralphaifname")

PAT_INSERT_DECL(etheralphaifname_pat_insert)
{
  t_status	status;

  assert(sz_pat.insert_proc);
  if ((status = sz_pat.insert_proc(NULL,
				   buf,
				   len,
				   value)) < 0)
    return (status);
  if (etheralpha_fd >= 0)
    etheralpha_close(etheralpha_fd);
  if ((etheralpha_fd = etheralpha_open(etheralpha_ifname)) < 0)
    {
      err_print(ERR_XIP_OPEN,etheralpha_ifname);
      return (0);
    }
#ifdef DEBUG
  if (XIP_VERB(VERB_XIP_MISC))
    fprintf(stderr,"etheralpha_open %s, fd=%d\n",
	    etheralpha_ifname,
	    etheralpha_fd);
#endif
  return (0);
}

t_pat				etheralphaifname_pat = 
{
  etheralphaifname_pat_name,	/* t_pat_name_proc		*/
  NULL,				/* t_pat_off_proc		*/
  NULL,				/* t_pat_sub_proc		*/
  NULL,				/* t_pat_sum_proc		*/
  NULL,				/* t_pat_get_field_proc		*/
  NULL,				/* t_pat_set_field_proc		*/
  NULL,				/* t_pat_get_fields_proc	*/
  NULL,				/* t_pat_get_tmpl_proc		*/
  NULL,				/* t_pat_get_tmpl2_proc		*/
  NULL,				/* t_pat_has_opt_proc		*/
  NULL,				/* t_pat_adapt_len_proc		*/
  NULL,				/* t_pat_get_field_pat_proc	*/
  sz_pat_extract,		/* t_pat_extract_proc		*/
  etheralphaifname_pat_insert,	/* t_pat_insert_proc		*/
  NULL,				/* t_pat_get_choices_proc	*/
};

t_xip_var		etheralpha_vars[] = 
{
  {"etheralpha_ether_dst",
   (t_off)(&etheralpha_ether_dst),
   sizeof (etheralpha_ether_dst),
   &etheraddr_pat,
   NULL},
  {"etheralpha_ifname",
   (t_off)(&etheralpha_ifname),
   sizeof (etheralpha_ifname),
   &etheralphaifname_pat,
   NULL},
  NULL_XIP_VAR
};

t_xip_cf_box_entry	etheralpha_entries[] =
{
  {"etheralphaEtherDst","etheralpha_ether_dst"},
  {"etheralphaIfName",	"etheralpha_ifname"},
  NULL_XIP_CF_BOX_ENTRY
};

t_xip_cf_box_group	etheralpha_group = 
{"etheralphaGroup",	"etheralpha",	etheralpha_entries};

/* initializes alpha stuff.
   Note: registers a new packet bar title, a new method and new variables.
   Returns 0. If it is not able to open etheralpha_ifname (interface), it
   prints out an error message. */
t_status		xip_send_init(VOID_DECL)
{
  t_status		status;
  t_xip_method		xm;
  t_xip_pkt_bar		xpb;
  t_xip_var		*xv;
  
  xm.name = "send";
  xm.proc = etheralpha_method_proc_send;
  if ((status = xip_method_add(&xm)) < 0)
    return (status);
  xpb.label = "Send";
  xpb.methodstr = "send()";
  if ((status = xip_pkt_bar_add(&xpb)) < 0)
    return (status);
  xv = etheralpha_vars;
  while (xv->name)
    {
      if ((status = xip_var_add(xv)) < 0)
	return (status);
      xv++;
    }
  if ((status = xip_cf_box_group_add(&etheralpha_group)) < 0)
    return (status);
  return (0);
}
#endif

#ifdef HAVE_NO_SEND
t_status		xip_send_init(VOID_DECL)
{
  return (0);
}
#endif
